# compute freq. domain beamformer for a single freq
# Based on fbeamf_R.m by Paulo Felisberto
# Faro, Sab 11 Set 2021 22:51:53 WEST 
# Written by Orlando Camargo Rodriguez

from numpy import *
from pylab import *

nPh = 40
d   = 4 # Hydrophone spacing (OAEx Exp)
phi = 15*pi/180 # direction of simulated source (against array)
fs  = 750       #sampling frequency
t = arange(0,1,1.0/fs) #time
f  = 187.5            # signal frequency
c  = 1500  # sound speed in salt water
NFFT = 1024

# simulate acquired signal 
k0 = int( fix( f/fs*NFFT ) ) 

s = zeros((nPh,fs))
S = zeros((nPh,fs)) + 1j*zeros((nPh,fs))
Y = zeros( nPh ) + 1j*zeros( nPh )

for i in range(nPh):
    s[i,:] = sin( 2*pi*k0*fs/NFFT*( t + i*d*sin( phi )/c) ) #+7*randn(size(t));

#processing:
for i in range(nPh):
    Si = fft( s[i,:], NFFT )
    S[i,:] = Si[0:fs]

fmx = float(fs)/float(NFFT)*k0 #find the freq-bin where is the maximum (only one frequency is used)

Y   = S[:,k0+1]

phis = arange(-90,91,1)

nPhis = phis.size

B = zeros( nPhis ) + 1j*zeros( nPhis )

for i in range(nPhis):
   tau = d*sin( phis[i]*pi/180 )/c
   taus = arange( nPh )*tau
   exd = exp( 1j*2*pi*fmx*taus ) 
   B[i] = dot( Y , exd )

figure
plot(phis,abs(B))
grid(True)
show()
