# EOFs calculation
# Faro, Sab 11 Set 2021 22:52:58 WEST 
# Written by Orlando Camargo Rodriguez

from numpy import *
from scipy.io import *
from scipy import signal
from matplotlib.pyplot import *

data = loadmat("atlantico.mat") 

C = data["C"]
z = 1.0*squeeze( data["z"] )
zkm = -z/1000.0
Csize = C.shape
figure(1)
for i in range(Csize[1]):
    plot(C[:,i],zkm,linewidth=2)
xlabel('Velocidade de propagacao (m/s)')
ylabel('Profundidade (km)') 
title(r'Atlantico Norte: perfis de velocidade -70$^{\circ}$E,20-40$^{\circ}$N')
grid(True)

# Calculo do perfil medio: 

c0 = C.mean( axis=1 )

figure(2)
plot(c0,zkm,linewidth=2)
xlabel('Velocidade de propagacao (m/s)')
ylabel('Profundidade (km)')
title(r'Perfil medio')
grid(True)

# Calculo da perturbacao:

deltaC = zeros( Csize )

for i in range(Csize[1]):
    deltaC[:,i] = C[:,i] - c0

x = range(1,Csize[1]+1)

figure(3)
pcolor( x, zkm, deltaC )
colorbar()
xlabel('Numero de observacoes')
ylabel('Profundidade (km)')
title(r'$\delta$ C')
xlim(1,Csize[1]+1)
ylim(min(zkm),0)

# Decomposicao em valores singulares: 

U,s,V = linalg.svd( deltaC )

valores_proprios = s
variancia = valores_proprios*valores_proprios/sum( valores_proprios*valores_proprios )

figure(4)
subplot(141), plot( U[:,0], zkm ); title(r'EOF$_1$'), ylabel('Profundidade (km)'), grid(True)
subplot(142), plot( U[:,1], zkm ); title(r'EOF$_2$'), grid(True)
subplot(143), plot( U[:,2], zkm ); title(r'EOF$_3$'), grid(True)  
subplot(144), plot( U[:,3], zkm ); title(r'EOF$_4$'), grid(True)

n = range(1,Csize[0]+1)

figure(5)
stem( n, valores_proprios )
grid(True)
xlabel('i')
ylabel(r'$\lambda_i$')
title('Distribuicao de valores proprios')
xlim(0,Csize[0]+1)

figure(6)
stem( n, variancia )
grid(True)
xlabel('i')
ylabel('\lambda_i')
title('Variancia das EOFs')
axis([0,Csize[0]+1,0,1])

S       = zeros( Csize )
deltaCi = zeros( Csize )
Ui      = zeros( U.shape )
Vi      = zeros( V.shape )

# Importancia das EOFs: 
# vamos reconstruir progressivamente deltaC:

for i in range(5):
    Ui[:,i] = U[:,i]
    Vi[:,i] = V[i,:]
    S[i,i] = valores_proprios[i]
    deltaCi = array( mat(Ui)*mat(S)*mat(Vi.T) )
    figure(7)
    pcolor( deltaCi )
    pcolor( x, zkm, deltaCi )
    ylabel('Profundidade (km)')
    titulo = '$\delta$ C, Numero de EOFs = ' + str(i+1)
    title( titulo )
    xlim(1,Csize[1]+1)
    ylim(min(zkm),0)
    pause(0.5)
colorbar()

show()
