#*******************************************************************************
# Pekeris phase and group speed as function of frequency
# Faro, Sex 10 Set 2021 18:04:17 WEST 
# Written by Orlando Camargo Rodriguez
# Don't like it? Don't use it... 
#*******************************************************************************
from numpy import * 
from scipy import * 
from matplotlib.pyplot import *

def kpekeris(c1=None,rho1=None,c2=None,rho2=None,D=None,freq=None):
    
    kp = []

    w = 2*pi*freq
    k1 = w/c1
    k2 = w/c2

    nk = 1001
    k = linspace(k1,k2,nk)
    kxk = k*k

    r1 = sqrt( k1*k1 - kxk ) 
    r2 = sqrt( kxk - k2*k2 )
    s1 = sin( r1*D )
    c1 = cos( r1*D )

    f = s1*r2/rho2 + c1*r1/rho1

    for i in range(nk-1): 
   
        x1 = k[ i ]
        x2 = k[i+1]
        y1 = f[ i ]
        y2 = f[i+1]
        p = y1*y2
        x3  = (x1*y2-x2*y1)/(y2-y1)
     
        if p < 0:

           kp = append(kp,x3)

    return kp

D    =  100.0
rho1 =    1.0
rho2 =    1.5
c1   = 1500.0
c2   = 2000.0
nfreqs = 101
fmin = 100.0
fmax = 200.0
freqs = linspace(fmin,fmax,nfreqs)
w = 2*pi*freqs

k1 = zeros(nfreqs)
k2 = zeros(nfreqs)
k3 = zeros(nfreqs)
k4 = zeros(nfreqs)

for i in range(nfreqs):
    wi = 2*pi*freqs[i]
    k = kpekeris(c1,rho1,c2,rho2,D,freqs[i])
    km = k[0:4]
    k1[i] = km[0]
    k2[i] = km[1]
    k3[i] = km[2]
    k4[i] = km[3]

vf1 = w/k1
vf2 = w/k2
vf3 = w/k3
vf4 = w/k4
dw = diff( w )
vg1 = dw/diff( k1 ); vg1 = append(vg1,vg1[-1])
vg2 = dw/diff( k2 ); vg2 = append(vg2,vg2[-1])
vg3 = dw/diff( k3 ); vg3 = append(vg3,vg3[-1])
vg4 = dw/diff( k4 ); vg4 = append(vg4,vg4[-1])

figure(2)
plot(freqs,vf1,'k',linewidth=2)
plot(freqs,vf2,'k',linewidth=2)
plot(freqs,vf3,'k',linewidth=2)
plot(freqs,vf4,'k',linewidth=2)
plot(freqs,vg1,'k--',linewidth=2)
plot(freqs,vg2,'k--',linewidth=2)
plot(freqs,vg3,'k--',linewidth=2)
plot(freqs,vg4,'k--',linewidth=2)
xlabel('Frequency (Hz)',fontsize=18)
ylabel('Speed (m/s)',fontsize=18)
grid(True)

show()

