#*******************************************************************************
# TL with modes for the isovelocity waveguide
# Faro, Sex 10 Set 2021 17:58:07 WEST 
# Written by Orlando Camargo Rodriguez
# Don't like it? Don't use it... 
#*******************************************************************************
from numpy import * 
from scipy import * 
from matplotlib.pyplot import *

c    = 1500
freq =  20; w = 2*pi*freq; k = w/c; kxk = k*k
D    = 100
zs   =  25
nz  = 51
z   = linspace(0,D,nz)
nr = 201
rmax = 1000
r  = linspace(0,rmax,nr); r[0] = 1.0

M = int( floor( w*D/( c*pi ) + 0.5 ) )
m = arange(1,M+1)
kz = ( m - 0.5 )*pi/D
kh = sqrt( kxk - kz*kz )

p  = zeros((nz,nr)) + 1j*zeros((nz,nr))
pm = zeros((nz,nr)) + 1j*zeros((nz,nr))

[R,Z] = meshgrid(r,z)

p0 = sqrt( 8*pi )/D

for i in range(M):
    Zh  = sin( kz[i]*Z  )
    Zs  = sin( kz[i]*zs ) 
    Zi  = Zs*Zh 
    Ri  = exp( 1j*kh[i]*R )/sqrt( kh[i]*R )
    pm = Zi*Ri
    p = p + pm

p = p*p0

tl = -20*log10( abs(p) )

figure(1)
imshow(tl,extent=[r[0],r[-1],z[-1],z[0]], aspect='auto',cmap='jet_r')
colorbar()
xlabel(r'Range (m)',fontsize=16)
ylabel(r'Depth (m)',fontsize=16)
title('Isovelocity Problem - TL (dB)')

show() 

print("done.")
 
